@@ -24,7 +24,7 @@ import shlex
 import subprocess
 import sys
 from datetime import datetime, timezone  # pylint: disable=wrong-import-order
-from typing import Any, Dict, List, Optional, Union  # pylint: disable=wrong-import-order
+from typing import Any, Dict, List, Optional, Union, Tuple  # pylint: disable=wrong-import-order
 
 import psutil
 
@@ -49,13 +49,20 @@ def process_arguments(argv: List[str]) -> argparse.Namespace:
     group.add_argument(
         "--export", action="store_true", help="export credentials as environment variables")
     group.add_argument(
-        "--generate", action="store_true", help="generate credentials file from the input profile")
+        "--generate",
+        action="store_true",
+        help="generate credentials from the input profile, optionally stored in specified file")
     group.add_argument("--process", action="store_true")
     group.add_argument("--exec", action="store")
-    profile_from_envvar = os.environ.get("AWS_PROFILE", os.environ.get("AWS_DEFAULT_PROFILE", None))
+    profile_from_envvar = os.environ.get(
+        "AWS_PROFILE", os.environ.get(
+            "AWS_DEFAULT_PROFILE", "default"))
     parser.add_argument(
         "--profile", action="store", default=profile_from_envvar,
         help="the source profile to use for creating credentials")
+    parser.add_argument(
+        "--generatestdout", action="store_true",
+        help="outputs generated credentials to the console not saved to file")
     parser.add_argument(
         "--outprofile", action="store", default="default",
         help="the destination profile to save generated credentials")
@@ -83,10 +90,38 @@ def retrieve_attribute(profile: Dict[str, Any], tag: str) -> Any:
         Aws2WrapError: The tag was not present in the profile.
     """
     if tag not in profile:
-        raise Aws2WrapError(f"{tag!r} not in {profile!r} profile")
+        raise Aws2WrapError(f"{tag!r} not found in profile: {profile!r}")
     return profile[tag]
 
 
+def readline_generator(file_handle):
+    """Support mocked reading of config with Python 3.6"""
+    line = file_handle.readline()
+    while line:
+        yield line
+        line = file_handle.readline()
+
+
+def read_aws_config() -> Tuple[configparser.ConfigParser, str]:
+    """Read the AWS config from the appropriate file"""
+    aws_config_file = os.environ.get("AWS_CONFIG_FILE")
+    if aws_config_file:
+        config_path = os.path.abspath(aws_config_file)
+    else:
+        config_path = os.path.abspath(os.path.expanduser("~/.aws/config"))
+    config = configparser.ConfigParser()
+    # Mocking "open" in Python 3.6 doesn't work with ConfigParser.
+    # It is suspected that this is because the reading mechanism
+    # iterates on the file handle and not by calling readline().
+    # If this package ever stops supporting Python 3.6, the following
+    # block of code can be replaced with:
+    # config.read(config_path)
+    # and the whole of "def readline_generator" removed
+    with open(config_path, mode="r", encoding="utf-8") as conf:
+        config.read_file(readline_generator(conf))
+    return config, config_path
+
+
 def retrieve_profile(profile_name: str) -> ProfileDef:
     """Find the AWS Config profile matching the specified profile name.
 
@@ -97,18 +132,12 @@ def retrieve_profile(profile_name: str) -> ProfileDef:
     Raises:
         Aws2WrapError: The profile was not found in the config file.
     """
-    aws_config_file = os.environ.get("AWS_CONFIG_FILE")
-    if aws_config_file:
-        config_path = os.path.abspath(aws_config_file)
-    else:
-        config_path = os.path.abspath(os.path.expanduser("~/.aws/config"))
-    config = configparser.ConfigParser()
-    config.read(config_path)
+    config, config_path = read_aws_config()
 
     if profile_name == "default":
         section_name = "default"
     else:
-        section_name = "profile %s" % profile_name
+        section_name = f"profile {profile_name}"
 
     # Look for the required profile
     if section_name not in config:
@@ -140,7 +169,7 @@ def retrieve_token_from_file(
     Returns:
         The access token if matched and not expired, otherwise None.
     """
-    with open(filename, "r") as json_file:
+    with open(filename, mode="r", encoding="utf-8") as json_file:
         blob = json.load(json_file)
     if ("startUrl" not in blob or
             blob["startUrl"] != sso_start_url or
@@ -224,7 +253,8 @@ def get_role_credentials(profile: ProfileDef) -> Dict[str, Any]:
             stdout=subprocess.PIPE
         )
     except subprocess.CalledProcessError as error:
-        print(error.stderr.decode(), file=sys.stderr)
+        if error.stderr is not None:
+            print(error.stderr.decode(), file=sys.stderr)
         raise Aws2WrapError(f"Please login with 'aws sso login --profile={profile_name}'") from None
 
     output = json.loads(result.stdout)
@@ -267,7 +297,7 @@ def get_assumed_role_credentials(profile: ProfileDef) -> Dict[str, Dict[str, str
         role_session_name = retrieve_attribute(profile, "role_session_name")
     else:
         unix_time = int(datetime.now().timestamp())
-        role_session_name = "botocore-session-%d" % unix_time
+        role_session_name = f"botocore-session-{unix_time}"
 
     # AssumeRole using source credentials
     try:
@@ -284,7 +314,8 @@ def get_assumed_role_credentials(profile: ProfileDef) -> Dict[str, Dict[str, str
             env=env,
         )
     except subprocess.CalledProcessError as error:
-        print(error.stderr.decode(), file=sys.stderr)
+        if error.stderr is not None:
+            print(error.stderr.decode(), file=sys.stderr)
         role_arn = retrieve_attribute(profile, "role_arn")
         raise Aws2WrapError(f"Failed to assume-role {role_arn!r}") from None
 
@@ -328,7 +359,7 @@ def process_cred_generation(  # pylint: disable=too-many-arguments
         "aws_secret_access_key": secret_access_key,
         "aws_session_token": session_token
     }
-    with open(credentialsfile, "w") as file:
+    with open(credentialsfile, mode="w", encoding="utf-8") as file:
         config.write(file)
 
     config = configparser.ConfigParser()
@@ -338,13 +369,13 @@ def process_cred_generation(  # pylint: disable=too-many-arguments
         new_config = {
             "region": retrieve_attribute(profile, "region")
         }
-    config[outprofile] = new_config
-    with open(configfile, "w") as file:
+    config[f"profile {outprofile}"] = new_config
+    with open(configfile, mode="w", encoding="utf-8") as file:
         config.write(file)
 
-    print("Credentials written to %s" % credentialsfile)
-    print("Configuration written to %s" % configfile)
-    print("The credentials will expire at %s" % expiration)
+    print(f"Credentials written to {credentialsfile}")
+    print(f"Configuration written to {configfile}")
+    print(f"The credentials will expire at {expiration}")
 
 
 def run_command(
@@ -404,19 +435,19 @@ def export_credentials(
     is_powershell = bool(re.fullmatch(r'pwsh|pwsh.exe|powershell.exe', shell_name))
 
     if is_powershell:
-        print("$ENV:AWS_ACCESS_KEY_ID=\"%s\"" % access_key)
-        print("$ENV:AWS_SECRET_ACCESS_KEY=\"%s\"" % secret_access_key)
-        print("$ENV:AWS_SESSION_TOKEN=\"%s\"" % session_token)
+        print(f"$ENV:AWS_ACCESS_KEY_ID=\"{access_key}\"")
+        print(f"$ENV:AWS_SECRET_ACCESS_KEY=\"{secret_access_key}\"")
+        print(f"$ENV:AWS_SESSION_TOKEN=\"{session_token}\"")
         # If region is specified in profile, also export AWS_DEFAULT_REGION
         if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
-            print("$ENV:AWS_DEFAULT_REGION=\"%s\"" % retrieve_attribute(profile, "region"))
+            print(f"$ENV:AWS_DEFAULT_REGION=\"{retrieve_attribute(profile, 'region')}\"")
     else:
-        print("export AWS_ACCESS_KEY_ID=%s" % access_key)
-        print("export AWS_SECRET_ACCESS_KEY=%s" % secret_access_key)
-        print("export AWS_SESSION_TOKEN=%s" % session_token)
+        print(f"export AWS_ACCESS_KEY_ID={access_key}")
+        print(f"export AWS_SECRET_ACCESS_KEY={secret_access_key}")
+        print(f"export AWS_SESSION_TOKEN={session_token}")
         # If region is specified in profile, also export AWS_DEFAULT_REGION
         if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
-            print("export AWS_DEFAULT_REGION=%s" % retrieve_attribute(profile, "region"))
+            print(f"export AWS_DEFAULT_REGION={retrieve_attribute(profile, 'region')}")
 
 
 def main(argv: Optional[List[str]]=None) -> int:
@@ -425,10 +456,6 @@ def main(argv: Optional[List[str]]=None) -> int:
         argv = sys.argv
     args = process_arguments(argv)
     try:
-        if args.profile is None:
-            raise Aws2WrapError(
-                "Please specify profile name by --profile or environment variable AWS_PROFILE")
-
         profile = retrieve_profile(args.profile)
 
         if "source_profile" in profile:
@@ -445,7 +472,11 @@ def main(argv: Optional[List[str]]=None) -> int:
             # On Windows, parent process is aws2-wrap.exe, in unix it's the shell
             export_credentials(access_key, secret_access_key, session_token, profile)
         elif args.generate:
-            if args.outprofile is not None:
+            if args.generatestdout or args.outprofile is None:
+                print("aws_access_key_id =", access_key)
+                print("aws_secret_access_key =", secret_access_key)
+                print("aws_session_token =", session_token)
+            else:
                 process_cred_generation(
                     args.credentialsfile, args.configfile, expiration, args.outprofile,
                     access_key, secret_access_key, session_token, profile)
